#include "common.h"
#include "cpu_conf.h"
#include "NRF.h"


uint8_t spi_send_recv(uint8_t data) 
{
  while (!(SPI1->SR & SPI_SR_TXE));      //  
  SPI1->DR = data;                       //   
  while (!(SPI1->SR & SPI_SR_RXNE));     //  
  return (SPI1->DR);		        				 //  
}


void radio_assert_ce(void){nrf_ce_high;} 


void radio_deassert_ce(void){nrf_ce_low;}

void csn_assert(void) {nrf_cs_low;}

void csn_deassert(void) {nrf_cs_high;}

void radio_init(void) 
{
	
	// IRQ
	GPIOB->CRL &=~GPIO_CRL_MODE0;
	GPIOB->CRL &=~GPIO_CRL_CNF0_0;
	GPIOB->CRL |= GPIO_CRL_CNF0_1;
	GPIOB->BSRR|= GPIO_BSRR_BR0;
	// NSS
	GPIOA->CRL |= GPIO_CRL_MODE4;  
	GPIOA->CRL &=~GPIO_CRL_CNF4;
	GPIOA->BSRR|= GPIO_BSRR_BS4;
	//CE	
	GPIOA->CRL |= GPIO_CRL_MODE3;  
	GPIOA->CRL &= ~GPIO_CRL_CNF3;
	GPIOA->BSRR|= GPIO_BSRR_BR3;
	
  csn_deassert();
  radio_deassert_ce();
 // spi_init();
	SPI1_init ();
}

//   cmd,   count  ,     buf,   
uint8_t radio_read_buf(uint8_t cmd, uint8_t *buf, uint8_t count) 
{
	uint8_t status=0;
	csn_assert();
  status = spi_send_recv(cmd);
  while (count--) {
    *(buf++) = spi_send_recv(0xFF);
  }
  csn_deassert();
  return status;
}

//   cmd,   count    buf, ?  
uint8_t radio_write_buf(uint8_t cmd, uint8_t * buf, uint8_t count) 
{
  uint8_t status=0;
	csn_assert();
  status = spi_send_recv(cmd);
  while (count--) {
    spi_send_recv(*(buf++));
  }
  csn_deassert();
	_delay_10us(3);
  return status;
}

//     reg ( 0  31)   
uint8_t radio_readreg(uint8_t reg) 
{
  uint8_t answ=0;
	csn_assert();
  spi_send_recv((reg & 31) | R_REGISTER);
  answ = spi_send_recv(0xFF);
  csn_deassert();
	_delay_10us(3);
  return answ;
}

//     reg ( 0  31),   
uint8_t radio_writereg(uint8_t reg, uint8_t val) 
{
  uint8_t status=0;
	csn_assert();
	status = spi_send_recv((reg & 31) | W_REGISTER);
  spi_send_recv(val);
  csn_deassert();
	_delay_10us(3);
  return status;
}

//   count   reg ( 0  31)      buf,
//   
uint8_t radio_readreg_buf(uint8_t reg, uint8_t * buf, uint8_t count) 
{
  return radio_read_buf((reg & 31) | R_REGISTER, buf, count);
}

//   count   buf    reg ( 0  31) 
//   
uint8_t radio_writereg_buf(uint8_t reg, uint8_t * buf, uint8_t count) 
{
  return radio_write_buf((reg & 31) | W_REGISTER, buf, count);
}

//     FIFO  
uint8_t radio_read_rx_payload_width(void) 
{
	uint8_t answ=0;
  csn_assert();
  spi_send_recv(R_RX_PL_WID);
  answ = spi_send_recv(0xFF);
  csn_deassert();
	_delay_10us(3);
  return answ;
}

//  .   
uint8_t radio_cmd(uint8_t cmd)
{
  uint8_t status=0;
	csn_assert();
  status = spi_send_recv(cmd);
  csn_deassert();
	_delay_10us(3);
  return status;
}

//  1,    IRQ  
uint8_t radio_is_interrupt(void)
{
 // return (RADIO_PIN & RADIO_IRQ) ? 0 : 1;
	 return (GPIOA->IDR & nrf_IRQ) ? 0 : 1;
}

// ?   . 1-. 0-
uint8_t radio_start(void) 
{
/*
	// TX board config
	uint8_t cnt=0;
	uint8_t self_addr[] = {0xE7, 0xE7, 0xE7, 0xE7, 0xE7}; //  
  uint8_t remote_addr[] = {0xC2, 0xC2, 0xC2, 0xC2, 0xC2}; //   
  uint8_t chan =3; //   (  0 - 125)
	uint8_t tmp=0;

  radio_deassert_ce();
  for(cnt = 100;;) 
	{
    radio_writereg(CONFIG, (1 << EN_CRC) | (1 << CRCO) | (1 << PRIM_RX)); //  
		_delay_ms(2);
		tmp=radio_readreg(CONFIG);
    if (tmp == ((1 << EN_CRC) | (1 << CRCO) | (1 << PRIM_RX))) 
      break;
    //             
    if (!cnt--)
      return 0; //   100        
		_delay_ms(1);
  }
	
  radio_writereg(EN_AA, (1 << ENAA_P1)); //      1
  radio_writereg(EN_RXADDR, (1 << ERX_P0) | (1 << ERX_P1)); // ?  0  1
  radio_writereg(SETUP_AW, SETUP_AW_5BYTES_ADDRESS); //    5 
  radio_writereg(SETUP_RETR, SETUP_RETR_DELAY_250MKS | SETUP_RETR_UP_TO_2_RETRANSMIT); 
  radio_writereg(RF_CH, chan); //   
  radio_writereg(RF_SETUP, RF_SETUP_1MBPS | RF_SETUP_0DBM); //  1 ./   0dBm
  
  radio_writereg_buf(RX_ADDR_P0, &remote_addr[0], 5); //     0
  radio_writereg_buf(TX_ADDR, &remote_addr[0], 5);

  radio_writereg_buf(RX_ADDR_P1, &self_addr[0], 5);
  
  radio_writereg(RX_PW_P0, 0);
  radio_writereg(RX_PW_P1, 32); 
  radio_writereg(DYNPD, (1 << DPL_P0) | (1 << DPL_P1)); //     0  1
  radio_writereg(FEATURE, 0x04); //     
	
  radio_writereg(CONFIG, (1 << EN_CRC) | (1 << CRCO) | (1 << PWR_UP) | (1 << PRIM_RX)); //  
	_delay_ms(2);
	tmp=radio_readreg(CONFIG);
	
  return (tmp == ((1 << EN_CRC) | (1 << CRCO) | (1 << PWR_UP) | (1 << PRIM_RX)))? 1 : 0;
*/
	uint8_t cnt=0;
	uint8_t self_addr[] = {0xE7, 0xE7, 0xE7, 0xE7, 0xE7}; //  
  uint8_t remote_addr[] = {0xC2, 0xC2, 0xC2, 0xC2, 0xC2}; //   
  uint8_t chan =3; //   (  0 - 125)
	uint8_t tmp=0;

  radio_deassert_ce();
  for(cnt = 100;;) 
	{
    radio_writereg(CONFIG, (1 << EN_CRC) | (1 << CRCO) | (1 << PRIM_RX)); //  
		_delay_ms(2);
		tmp=radio_readreg(CONFIG);
    if (tmp == ((1 << EN_CRC) | (1 << CRCO) | (1 << PRIM_RX))) 
      break;
    //             
    if (!cnt--)
      return 0; //   100        
		_delay_ms(1);
  }
	
  radio_writereg(EN_AA, (1 << ENAA_P1)); //      1
  radio_writereg(EN_RXADDR, (1 << ERX_P0) | (1 << ERX_P1)); // ?  0  1
  radio_writereg(SETUP_AW, SETUP_AW_5BYTES_ADDRESS); //    5 
  radio_writereg(SETUP_RETR, SETUP_RETR_DELAY_250MKS | SETUP_RETR_UP_TO_2_RETRANSMIT); 
  radio_writereg(RF_CH, chan); //   
  radio_writereg(RF_SETUP, RF_SETUP_1MBPS | RF_SETUP_0DBM); //  1 ./   0dBm
  
  radio_writereg_buf(RX_ADDR_P0, remote_addr, 5); //     0
  radio_writereg_buf(TX_ADDR, self_addr, 5);

  radio_writereg_buf(RX_ADDR_P1, self_addr, 5);
  
  radio_writereg(RX_PW_P0, 32);
  radio_writereg(RX_PW_P1, 32); 
  radio_writereg(DYNPD, (1 << DPL_P0) | (1 << DPL_P1)); //     0  1
  radio_writereg(FEATURE, 0x04); //     
	
  radio_writereg(CONFIG, (1 << EN_CRC) | (1 << CRCO) | (1 << PWR_UP) | (1 << PRIM_RX)); //  
	_delay_ms(2);
	tmp=radio_readreg(CONFIG);
	
  return (tmp == ((1 << EN_CRC) | (1 << CRCO) | (1 << PWR_UP) | (1 << PRIM_RX)))? 1 : 0;

}


//     
// buf -   , size -   ( 1  32)
uint8_t send_data(uint8_t * buf, uint8_t size) 
{
  uint8_t conf=0;
	uint8_t status=0;

	radio_deassert_ce(); //       
  conf = radio_readreg(CONFIG);
  if (!(conf & (1 << PWR_UP))) //       
    return 0; 
  status = radio_writereg(CONFIG, conf & ~(1 << PRIM_RX)); //   PRIM_RX

  if (status & (1 << TX_FULL_STATUS))  //         
    return 0;
  radio_write_buf(W_TX_PAYLOAD, buf, size); //    
  radio_assert_ce(); //   CE    
  _delay_10us(4); //   10
  radio_deassert_ce();
  return 1;
}

void on_send_error(void) 
{
	
}
void on_packet(uint8_t * buf, uint8_t size) 
{

}


void check_radio(void) 
{
	uint8_t status=0;
	uint8_t conf=0;
	uint8_t protect=0;
	if (!radio_is_interrupt()) //      
   return;
   status = radio_cmd(NOP);
  radio_writereg(STATUS, status); //        
  
  if (status & ((1 << TX_DS) | (1 << MAX_RT))) { //      
    if (status & (1 << MAX_RT)) { //     
      radio_cmd(FLUSH_TX); //     
      on_send_error(); //  
    } 
    if (!(radio_readreg(FIFO_STATUS) & (1 << TX_EMPTY))) { //       
      radio_assert_ce(); //   CE    
     _delay_10us(4);//   10
      radio_deassert_ce();
    } else {
      conf = radio_readreg(CONFIG);
      radio_writereg(CONFIG, conf | (1 << PRIM_RX)); //   PRIM_RX: 
      radio_assert_ce(); //    CE    
    }
  }
  protect = 4; //   FIFO     3 .    
  while (((status & (7 << RX_P_NO)) != (7 << RX_P_NO)) && protect--) { //      
    uint8_t l = radio_read_rx_payload_width(); //   
    if (l > 32) { //     
      radio_cmd(FLUSH_RX); 
    } else { 
      uint8_t buf[32]; //    
      radio_read_buf(R_RX_PAYLOAD, &buf[0], l); //  
      if ((status & (7 << RX_P_NO)) == (1 << RX_P_NO)) { //  datapipe 1 
        on_packet(&buf[0], l); //    
      }
    }         
    status = radio_cmd(NOP);
  }
}

